@@ -185,7 +185,15 @@ OENTRY opcodlst_1[] = {
   { "##mul.aa",  S(AOP),0,    2,      "a",    "aa",   NULL,   mulaa   },
   { "##div.aa",  S(AOP),0,    2,      "a",    "aa",   NULL,   divaa   },
   { "##mod.aa",  S(AOP),0,    2,      "a",    "aa",   NULL,   modaa   },
-  { "divz",   0xfffc                                                      },
+  { "##addin.i", S(ASSIGN),0, 1,      "i",    "i",    addin,  NULL    },
+  { "##addin.k", S(ASSIGN),0, 2,      "k",    "k",    NULL,   addin   },
+  { "##addin.K", S(ASSIGN),0, 2,      "a",    "k",    NULL,   addinak },
+  { "##addin.a", S(ASSIGN),0, 2,      "a",    "a",    NULL,   addina  },
+  { "##subin.i", S(ASSIGN),0, 1,      "i",    "i",    subin,  NULL    },
+  { "##subin.k", S(ASSIGN),0, 2,      "k",    "k",    NULL,   subin   },
+  { "##subin.K", S(ASSIGN),0, 2,      "a",    "k",    NULL,   subinak },
+  { "##subin.a", S(ASSIGN),0, 2,      "a",    "a",    NULL,   subina  },
+  //{ "divz",   0xfffc                                                      },
   { "divz.ii", S(DIVZ),0,   1,      "i",    "iii",  divzkk, NULL,   NULL    },
   { "divz.kk", S(DIVZ),0,   2,      "k",    "kkk",  NULL,   divzkk, NULL    },
   { "divz.ak", S(DIVZ),0,   2,      "a",    "akk",  NULL,   divzak  },
@@ -368,38 +376,38 @@ OENTRY opcodlst_1[] = {
   /* { "table",  0xffff, TR                                                   }, */
   /* { "tablei", 0xffff, TR                                                   }, */
   /* { "table3", 0xffff, TR                                                   }, */
-  { "table.i",  S(TABL),0, 1,      "i",    "iiooo",(SUBR)tabler_init       },
-  { "table.k",  S(TABL),0, 3,      "k",    "xiooo",(SUBR)tabl_setup,
+  { "table.i",  S(TABL),TR, 1,      "i",    "iiooo",(SUBR)tabler_init       },
+  { "table.k",  S(TABL),TR, 3,      "k",    "xiooo",(SUBR)tabl_setup,
     (SUBR)tabler_kontrol        },
-  { "table.a",  S(TABL),0, 3,      "a",    "xiooo",(SUBR)tabl_setup,
+  { "table.a",  S(TABL),TR, 3,      "a",    "xiooo",(SUBR)tabl_setup,
     (SUBR)tabler_audio                                                      },
-  { "tablei.i", S(TABL),0, 1,      "i",    "iiooo",(SUBR)tableir_init       },
-  { "tablei.k", S(TABL),0, 3,      "k",    "xiooo",(SUBR)tabl_setup,
+  { "tablei.i", S(TABL),TR, 1,      "i",    "iiooo",(SUBR)tableir_init       },
+  { "tablei.k", S(TABL),TR, 3,      "k",    "xiooo",(SUBR)tabl_setup,
     (SUBR)tableir_kontrol                                                   },
-  { "tablei.a", S(TABL),0, 3,      "a",    "xiooo",(SUBR)tabl_setup,
+  { "tablei.a", S(TABL),TR, 3,      "a",    "xiooo",(SUBR)tabl_setup,
     (SUBR)tableir_audio                                                     },
-  { "table3.i", S(TABL),0, 1,      "i",    "iiooo",(SUBR)table3r_init       },
-  { "table3.k", S(TABL),0, 3,      "k",    "xiooo",(SUBR)tabl_setup,
+  { "table3.i", S(TABL),TR, 1,      "i",    "iiooo",(SUBR)table3r_init       },
+  { "table3.k", S(TABL),TR, 3,      "k",    "xiooo",(SUBR)tabl_setup,
     (SUBR)table3r_kontrol                                                   },
-  { "table3.a", S(TABL),0, 3,      "a",    "xiooo",(SUBR)tabl_setup,
+  { "table3.a", S(TABL),TR, 3,      "a",    "xiooo",(SUBR)tabl_setup,
     (SUBR)table3r_audio                                                     },
   /* { "ptable",  0xffff, TR                                                  }, */
   /* { "ptablei", 0xffff, TR                                                  }, */
   /* { "ptable3", 0xffff, TR                                                  }, */
-  { "ptable.i",  S(TABLE),0, 1,     "i",    "iiooo",(SUBR)tabler_init       },
-  { "ptable.k",  S(TABLE),0, 3,     "k",    "xiooo",(SUBR)tabl_setup,
+  { "ptable.i",  S(TABLE),TR, 1,     "i",    "iiooo",(SUBR)tabler_init       },
+  { "ptable.k",  S(TABLE),TR, 3,     "k",    "xiooo",(SUBR)tabl_setup,
     (SUBR)tabler_kontrol                                                    },
-  { "ptable.a",  S(TABLE),0, 3,     "a",    "xiooo",(SUBR)tabl_setup,
+  { "ptable.a",  S(TABLE),TR, 3,     "a",    "xiooo",(SUBR)tabl_setup,
     (SUBR)tabler_audio                                                      },
-  { "ptablei.i", S(TABLE),0, 1,     "i",    "iiooo",(SUBR)tableir_init      },
-  { "ptablei.k", S(TABLE),0, 3,     "k",    "xiooo",(SUBR)tabl_setup,
+  { "ptablei.i", S(TABLE),TR, 1,     "i",    "iiooo",(SUBR)tableir_init      },
+  { "ptablei.k", S(TABLE),TR, 3,     "k",    "xiooo",(SUBR)tabl_setup,
     (SUBR)tableir_kontrol                                                   },
-  { "ptablei.a", S(TABLE),0, 3,     "a",    "xiooo",(SUBR)tabl_setup,
+  { "ptablei.a", S(TABLE),TR, 3,     "a",    "xiooo",(SUBR)tabl_setup,
     (SUBR)tableir_audio                                                     },
-  { "ptable3.i", S(TABLE),0, 1,     "i",    "iiooo",(SUBR)table3r_init      },
-  { "ptable3.k", S(TABLE),0, 3,     "k",    "xiooo",(SUBR)tabl_setup,
+  { "ptable3.i", S(TABLE),TR, 1,     "i",    "iiooo",(SUBR)table3r_init      },
+  { "ptable3.k", S(TABLE),TR, 3,     "k",    "xiooo",(SUBR)tabl_setup,
     (SUBR)table3r_kontrol                                                   },
-  { "ptable3.a", S(TABLE),0, 3,     "a",    "xiooo",(SUBR)tabl_setup,
+  { "ptable3.a", S(TABLE),TR, 3,     "a",    "xiooo",(SUBR)tabl_setup,
     (SUBR)table3r_audio         },
   { "oscil1", S(OSCIL1), TR, 3,     "k",    "ikij", ko1set, kosc1          },
   { "oscil1i",S(OSCIL1), TR, 3,     "k",    "ikij", ko1set, kosc1i         },
@@ -670,12 +678,12 @@ OENTRY opcodlst_1[] = {
   { "filepeak.i", S(SNDINFOPEAK),0, 1, "i",   "io",   filepeak, NULL, NULL       },
   { "filevalid.i", S(FILEVALID),0, 1,  "i",   "i",    filevalid, NULL, NULL      },
   /*  { "nlalp", S(NLALP),0,     3,     "a",  "akkoo", nlalp_set, nlalp }, */
-  { "ptableiw",  S(TABLEW),TW, 1,   "", "iiiooo", (SUBR)tablew_init, NULL, NULL},
-  { "ptablew.kk", S(TABLEW),0,  3,  "", "kkiooo",(SUBR)tabl_setup,
+  { "ptableiw",  S(TABLEW),TW|_QQ, 1, "", "iiiooo", (SUBR)tablew_init, NULL, NULL},
+  { "ptablew.kk", S(TABLEW),TB,  3,  "", "kkiooo",(SUBR)tabl_setup,
     (SUBR)tablew_kontrol, NULL          },
-  { "ptablew.aa", S(TABLEW),0,  3,  "", "aaiooo",(SUBR)tabl_setup,
+  { "ptablew.aa", S(TABLEW),TB,  3,  "", "aaiooo",(SUBR)tabl_setup,
     (SUBR)tablew_audio               },
-  { "tableiw",  S(TABL),TW, 1,    "",   "iiiooo", (SUBR)tablew_init, NULL, NULL},
+  { "tableiw",  S(TABL),TW|_QQ, 1, "",   "iiiooo", (SUBR)tablew_init, NULL, NULL},
   { "tablew",  S(TABL),TW, 1,    "",   "iiiooo", (SUBR)tablew_init, NULL, NULL},
   { "tablew.kk", S(TABL),TW,  3,    "", "kkiooo",(SUBR)tabl_setup,
     (SUBR)tablew_kontrol, NULL          },
@@ -831,9 +839,9 @@ OENTRY opcodlst_1[] = {
   { "pvsftw",   S(PVSFTW),  TW, 3,    "k",   "fio",    pvsftwset, pvsftw, NULL  },
   { "pvsftr",   S(PVSFTR),TR, 3,    "",    "fio",    pvsftrset, pvsftr, NULL  },
   { "pvsinfo",  S(PVSINFO),0, 1,    "iiii","f",      pvsinfo, NULL, NULL    },
-  { "octave",   0xffff                                                      },
-  { "semitone", 0xffff                                                      },
-  { "cent",     0xffff                                                      },
+  /* { "octave",   0xffff                                                      }, */
+  /* { "semitone", 0xffff                                                      }, */
+  /* { "cent",     0xffff                                                      }, */
   { "octave.i", S(EVAL),0,    1,    "i",    "i",     powoftwo               },
   { "octave.k", S(EVAL),0,    2,    "k",    "k",     NULL,  powoftwo        },
   { "octave.a", S(EVAL),0,    2,    "a",    "a",     NULL, powoftwoa  },
@@ -843,7 +851,7 @@ OENTRY opcodlst_1[] = {
   { "cent.i",   S(EVAL),0,    1,    "i",    "i",     cent                   },
   { "cent.k",   S(EVAL),0,    2,    "k",    "k",     NULL,  cent            },
   { "cent.a",   S(EVAL),0,    2,    "a",    "a",     NULL, acent      },
-  { "db",       0xffff                                                      },
+  //{ "db",       0xffff                                                      },
   { "db.i",     S(EVAL),0,    1,    "i",    "i",     db                     },
   { "db.k",     S(EVAL),0,    2,    "k",    "k",     NULL, db               },
   { "db.a",     S(EVAL),0,    2,    "a",    "a",     NULL, dba        },
@@ -873,25 +881,25 @@ OENTRY opcodlst_1[] = {
   { "midipitchbend", S(MIDIPITCHBEND),0,3, "", "xoh",
     midipitchbend, midipitchbend },
   { "mididefault", S(MIDIDEFAULT),0, 3, "", "xx",   mididefault, mididefault },
-  { "invalue",   0xFFFF,   _CR,    0,   NULL,   NULL, NULL, NULL },
-  { "invalue.k", S(INVAL),0, 3, "k", "i",   (SUBR) invalset,(SUBR)  kinval, NULL },
-  { "invalue.i", S(INVAL),0, 1, "i", "i",   (SUBR) invalsetgo, NULL, NULL },
-  { "invalue.kS", S(INVAL),0, 3, "k", "S", (SUBR) invalset_S,(SUBR)  kinval, NULL },
-  { "invalue.iS", S(INVAL),0, 1, "i", "S", (SUBR) invalsetSgo, NULL, NULL },
-  { "invalue.S", S(INVAL),0, 3, "S", "i",
-                                      (SUBR) invalset_string, (SUBR) kinvalS, NULL },
-  { "invalue.SS", S(INVAL),0, 3, "S", "S",
-                                    (SUBR) invalset_string_S, (SUBR) kinvalS, NULL },
-  { "outvalue", S(OUTVAL), _CW, 3, "", "ik", (SUBR) outvalset, (SUBR) koutval, NULL},
+  //{ "invalue",   0xFFFF,   _CR,    0,   NULL,   NULL, NULL, NULL },
+  { "invalue.k", S(INVAL),_CR, 3, "k", "i", (SUBR) invalset,(SUBR)kinval, NULL },
+  { "invalue.i", S(INVAL),_CR, 1, "i", "i", (SUBR) invalsetgo, NULL, NULL },
+  { "invalue.kS", S(INVAL),_CR, 3, "k", "S",(SUBR) invalset_S,(SUBR)kinval, NULL },
+  { "invalue.iS", S(INVAL),_CR, 1, "i", "S", (SUBR) invalsetSgo, NULL, NULL },
+  { "invalue.S", S(INVAL),_CR, 3, "S", "i",
+                                      (SUBR) invalset_string, (SUBR)kinvalS, NULL },
+  { "invalue.SS", S(INVAL),_CR, 3, "S", "S",
+                                    (SUBR) invalset_string_S, (SUBR)kinvalS, NULL },
+  { "outvalue", S(OUTVAL), _CW, 3, "", "ik", (SUBR) outvalset, (SUBR)koutval, NULL},
   { "outvalue.i", S(OUTVAL), _CW, 1, "", "ii", (SUBR) outvalsetgo, NULL, NULL},
   { "outvalue.k", S(OUTVAL), _CW, 3, "", "Sk",
-                                          (SUBR) outvalset_S, (SUBR) koutval, NULL},
+                                          (SUBR) outvalset_S, (SUBR)koutval, NULL},
   { "outvalue.Si", S(OUTVAL), _CW, 1, "", "Si",
                                           (SUBR) outvalsetSgo, NULL, NULL},
   { "outvalue.S", S(OUTVAL), _CW, 3, "", "iS",
-                                     (SUBR) outvalset_string, (SUBR) koutvalS, NULL},
+                                     (SUBR) outvalset_string, (SUBR)koutvalS, NULL},
   { "outvalue.SS", S(OUTVAL), _CW, 3, "", "SS",
-                                   (SUBR) outvalset_string_S, (SUBR) koutvalS, NULL},
+                                   (SUBR) outvalset_string_S, (SUBR)koutvalS, NULL},
   /* IV - Oct 20 2002 */
   { "subinstr", S(SUBINST),0, 3, "mmmmmmmm", "SN",  subinstrset_S, subinstr },
   { "subinstrinit", S(SUBINST),0, 1, "",    "SN",   subinstrset_S, NULL, NULL     },
@@ -899,12 +907,14 @@ OENTRY opcodlst_1[] = {
   { "subinstrinit.i", S(SUBINST),0, 1, "",    "iN",   subinstrset, NULL, NULL     },
   { "nstrnum", S(NSTRNUM),0, 1,     "i",    "S",    nstrnumset_S, NULL, NULL      },
   { "nstrnum.i", S(NSTRNUM),0, 1,     "i",    "i",    nstrnumset, NULL, NULL      },
-  { "turnoff2",   0xFFFB,   _CW,    0, NULL,   NULL,   NULL, NULL, NULL          },
-  { "turnoff2.S",S(TURNOFF2),0,2,     "",     "Skk",  NULL, turnoff2S, NULL     },
-  { "turnoff2.c",S(TURNOFF2),0,2,     "",     "ikk",  NULL, turnoff2k, NULL     },
-  { "turnoff2.k",S(TURNOFF2),0,2,     "",     "kkk",  NULL, turnoff2k, NULL     },
-  { "turnoff2.i",S(TURNOFF2),0,2,     "",     "ikk",  NULL, turnoff2k, NULL     },
-  { "turnoff2.r",S(TURNOFF2),0,2,     "",     "ikk",  NULL, turnoff2k, NULL     },
+  { "nstrstr", S(NSTRSTR),0, 1,       "S",    "i",    nstrstr, NULL, NULL      },
+  { "nstrstr.k", S(NSTRSTR),0, 2,     "S",    "k",    NULL, nstrstr, NULL      },
+  //{ "turnoff2",   0xFFFB,   _CW,    0, NULL,   NULL,   NULL, NULL, NULL          },
+  { "turnoff2.S",S(TURNOFF2),_CW,2,     "",     "Skk",  NULL, turnoff2S, NULL     },
+  { "turnoff2.c",S(TURNOFF2),_CW,2,     "",     "ikk",  NULL, turnoff2k, NULL     },
+  { "turnoff2.k",S(TURNOFF2),_CW,2,     "",     "kkk",  NULL, turnoff2k, NULL     },
+  { "turnoff2.i",S(TURNOFF2),_CW,2,     "",     "ikk",  NULL, turnoff2k, NULL     },
+  { "turnoff2.r",S(TURNOFF2),_CW,2,     "",     "ikk",  NULL, turnoff2k, NULL     },
   { "cngoto", S(CGOTO),0,   3,      "",     "Bl",   ingoto, kngoto, NULL     },
   { "cnkgoto", S(CGOTO),0,   2,      "",     "Bl",   NULL,  kngoto, NULL     },
   { "cingoto", S(CGOTO),0,   1,      "",     "Bl",   ingoto, NULL, NULL     },
@@ -1034,12 +1044,14 @@ OENTRY opcodlst_1[] = {
 #ifdef HAVE_CURL
   {  "strfromurl", S(STRCPY_OP), 0, 1, "S", "S", (SUBR) str_from_url     },
 #endif
-  {  "changed.S", S(STRINDEX_OP),0, 3, "k",   "S",
+  {  "changed.S", S(STRCHGD),_QQ, 3, "k",   "S",
      (SUBR) str_changed, (SUBR) str_changed_k, NULL       },
-  { "loop_lt",   0xfffb                                                  },
-  { "loop_le",   0xfffb                                                  },
-  { "loop_gt",   0xfffb                                                  },
-  { "loop_ge",   0xfffb                                                  },
+  {  "changed2.S", S(STRCHGD),0, 3, "k",   "S",
+     (SUBR) str_changed, (SUBR) str_changed_k, NULL       },
+  /* { "loop_lt",   0xfffb                                                  }, */
+  /* { "loop_le",   0xfffb                                                  }, */
+  /* { "loop_gt",   0xfffb                                                  }, */
+  /* { "loop_ge",   0xfffb                                                  }, */
   { "loop_lt.i", S(LOOP_OPS),0,  1,  "", "iiil", (SUBR) loop_l_i, NULL, NULL   },
   { "loop_le.i", S(LOOP_OPS),0,  1,  "", "iiil", (SUBR) loop_le_i, NULL, NULL  },
   { "loop_gt.i", S(LOOP_OPS),0,  1,  "", "iiil", (SUBR) loop_g_i, NULL, NULL   },
@@ -1049,17 +1061,17 @@ OENTRY opcodlst_1[] = {
   { "loop_gt.k", S(LOOP_OPS),0,  2,  "", "kkkl", NULL, (SUBR) loop_g_p, NULL   },
   { "loop_ge.k", S(LOOP_OPS),0,  2,  "", "kkkl", NULL, (SUBR) loop_ge_p, NULL  },
   { "chnget",      0xFFFF,    _CR                                             },
-  { "chnget.i",    S(CHNGET),0,           1,      "i",            "S",
+  { "chnget.i",    S(CHNGET),_CR,           1,      "i",            "S",
     (SUBR) chnget_opcode_init_i, NULL, NULL               },
-  { "chnget.k",    S(CHNGET),0,           3,      "k",            "S",
+  { "chnget.k",    S(CHNGET),_CR,           3,      "k",            "S",
     (SUBR) chnget_opcode_init_k, (SUBR) notinit_opcode_stub, NULL },
-  { "chnget.a",    S(CHNGET),0,           3,      "a",            "S",
+  { "chnget.a",    S(CHNGET),_CR,           3,      "a",            "S",
     (SUBR) chnget_opcode_init_a, (SUBR) notinit_opcode_stub },
-  { "chnget.S",    S(CHNGET),0,           3,      "S",            "S",
+  { "chnget.S",    S(CHNGET),_CR,           3,      "S",            "S",
     (SUBR) chnget_opcode_init_S, (SUBR) chnget_opcode_perf_S, NULL},
-  { "chngetks",    S(CHNGET),0,           2,      "S",            "S",
+  { "chngetks",    S(CHNGET),_CR,           2,      "S",            "S",
     NULL, (SUBR) chnget_opcode_perf_S, NULL},
-  { "chnset",      0xFFFB,              _CW                               },
+  //{ "chnset",      0xFFFB,              _CW                               },
   { "chnset.i",    S(CHNGET),_CW,          1,      "",             "iS",
     (SUBR) chnset_opcode_init_i, NULL, NULL               },
   //  { "chnset.r",    S(CHNGET),0,           1,      "",             "iS",
@@ -1084,7 +1096,7 @@ OENTRY opcodlst_1[] = {
     (SUBR) chn_a_opcode_init, NULL, NULL                  },
   { "chn_S",       S(CHN_OPCODE),      _CW, 1,      "",             "Si",
     (SUBR) chn_S_opcode_init, NULL, NULL                  },
-  { "chnexport",   0xFFFF,  0,    0,   NULL,   NULL, NULL, NULL },
+  //{ "chnexport",   0xFFFF,  0,    0,   NULL,   NULL, NULL, NULL },
   { "chnexport.i", S(CHNEXPORT_OPCODE),0, 1,      "i",            "Sioooo",
     (SUBR) chnexport_opcode_init, NULL, NULL              },
   { "chnexport.k", S(CHNEXPORT_OPCODE),0, 1,      "k",            "Sioooo",
@@ -1101,17 +1113,17 @@ OENTRY opcodlst_1[] = {
       { "chnsend",     S(CHNSEND),0,          3,      "",             "So",
       (SUBR) chnsend_opcode_init, (SUBR) notinit_opcode_stub, NULL },
   */
-  { "chano",       0xFFFD,  _CW, 0,      NULL, NULL, NULL, NULL },
-  { "chano.k",     S(ASSIGN),0,           2,      "",             "kk",
+  //{ "chano",       0xFFFD,  _CW, 0,      NULL, NULL, NULL, NULL },
+  { "chano.k",     S(ASSIGN),_CW,           2,      "",             "kk",
     NULL, (SUBR) chano_opcode_perf_k, NULL                },
-  { "chano.a",     S(ASSIGN),0,           2,      "",             "ak",
+  { "chano.a",     S(ASSIGN),_CW,           2,      "",             "ak",
     NULL, (SUBR) chano_opcode_perf_a                },
   { "pvsout",     S(FCHAN),0,           3,      "",             "fk",
     (SUBR) pvsout_init, (SUBR) pvsout_perf, NULL                        },
-  { "chani",      0xFFFF,  _CR,      0,   NULL, NULL, NULL, NULL },
-  { "chani.k",     S(ASSIGN),0,           2,      "k",            "k",
+  //{ "chani",      0xFFFF,  _CR,      0,   NULL, NULL, NULL, NULL },
+  { "chani.k",     S(ASSIGN),_CR,           2,      "k",            "k",
     NULL, (SUBR) chani_opcode_perf_k, NULL                },
-  { "chani.a",     S(ASSIGN),0,           2,      "a",            "k",
+  { "chani.a",     S(ASSIGN),_CR,           2,      "a",            "k",
     NULL, (SUBR) chani_opcode_perf_a                },
   { "pvsin",     S(FCHAN),0,           3,      "f",            "kooopo",
     (SUBR)  pvsin_init, (SUBR) pvsin_perf, NULL                  },
