@@ -39,7 +39,7 @@ static CS_NOINLINE void diskin2_read_buffer(CSOUND *csound,
 {
     MYFLT *tmp;
     int32_t nsmps;
-    int32_t   i;
+    int32_t i;
     IGN(csound);
     /* swap buffer pointers */
     tmp = p->buf;
@@ -315,7 +315,7 @@ static int32_t diskin2_init_(CSOUND *csound, DISKIN2 *p, int32_t stringname)
       /* skip initialisation if requested */
       if (p->SkipInit != FL(0.0))
         return OK;
-      fdclose(csound, &(p->fdch));
+      fd_close(csound, &(p->fdch));
     }
     /* set default format parameters */
     memset(&sfinfo, 0, sizeof(SF_INFO));
@@ -323,9 +323,12 @@ static int32_t diskin2_init_(CSOUND *csound, DISKIN2 *p, int32_t stringname)
     sfinfo.channels = p->nChannels;
     /* check for user specified sample format */
     n = MYFLT2LONG(*p->iSampleFormat);
-    if (UNLIKELY(n < 0 || n > 10))
-      return csound->InitError(csound, Str("diskin2: unknown sample format"));
-    sfinfo.format = diskin2_format_table[n];
+    if (n<0) {
+      n = -n;
+      if (UNLIKELY(n < 0 || n > 10))
+        return csound->InitError(csound, Str("diskin2: unknown sample format"));
+      sfinfo.format = diskin2_format_table[n];
+    }
     /* open file */
     /* FIXME: name can overflow with very long string */
     if (stringname==0){
@@ -559,12 +562,12 @@ int32_t diskin2_perf_synchronous(CSOUND *csound, DISKIN2 *p)
 {
     uint32_t offset = p->h.insdshead->ksmps_offset;
     uint32_t early  = p->h.insdshead->ksmps_no_end;
-    int nsmps = CS_KSMPS;
-    int chn, i, nn;
-    double  d, frac_d, x, c, v, pidwarp_d;
-    MYFLT   frac, a0, a1, a2, a3, onedwarp, winFact;
-    int32_t   ndx;
-    int32_t     wsized2, warp;
+    int      nsmps = CS_KSMPS;
+    int      chn, i, nn;
+    double   d, frac_d, x, c, v, pidwarp_d;
+    MYFLT    frac, a0, a1, a2, a3, onedwarp, winFact;
+    int32_t  ndx;
+    int32_t  wsized2, warp;
 
 
     if (UNLIKELY(p->fdch.fd == NULL) ) goto file_error;
@@ -749,9 +752,9 @@ int32_t diskin_file_read(CSOUND *csound, DISKIN2 *p)
     int32_t chn, chans = p->nChannels;
     double  d, frac_d, x, c, v, pidwarp_d;
     MYFLT   frac, a0, a1, a2, a3, onedwarp, winFact;
-    int32_t   ndx;
-    int32_t     wsized2, warp;
-    MYFLT  *aOut = (MYFLT *)p->aOut_buf; /* needs to be allocated */
+    int32_t ndx;
+    int32_t wsized2, warp;
+    MYFLT   *aOut = (MYFLT *)p->aOut_buf; /* needs to be allocated */
 
     if (UNLIKELY(p->fdch.fd == NULL) ) goto file_error;
     if (!p->initDone && !p->SkipInit) {
@@ -1032,7 +1035,7 @@ static int32_t sndo1set_(CSOUND *csound, void *pp, int32_t stringname)
     char    *sfname, *opname, name[1024];
     SNDCOM  *q;
     MYFLT   *ifilcod, *iformat;
-    int32_t     filetyp = TYP_RAW, format = csound->oparms_.outformat, nchns = 1;
+    int32_t filetyp = TYP_RAW, format = csound->oparms_.outformat, nchns = 1;
     SF_INFO sfinfo;
     //SNDOUTS *p = (SNDOUTS*) pp;
 
@@ -1157,9 +1160,9 @@ static CS_NOINLINE void diskin2_read_buffer_array(CSOUND *csound,
                                                   DISKIN2_ARRAY *p,
                                                   int32_t bufReadPos)
 {
-    MYFLT *tmp;
+    MYFLT   *tmp;
     int32_t nsmps;
-    int32_t   i;
+    int32_t i;
     IGN(csound);
     /* swap buffer pointers */
     tmp = p->buf;
@@ -1567,7 +1570,7 @@ static int32_t diskin2_init_array(CSOUND *csound, DISKIN2_ARRAY *p,
       /* skip initialisation if requested */
       if (p->SkipInit != FL(0.0))
         return OK;
-      fdclose(csound, &(p->fdch));
+      fd_close(csound, &(p->fdch));
     }
     // to handle raw files number of channels
     if (t->data) p->nChannels = t->sizes[0];
@@ -1577,9 +1580,12 @@ static int32_t diskin2_init_array(CSOUND *csound, DISKIN2_ARRAY *p,
     sfinfo.channels = p->nChannels;
     /* check for user specified sample format */
     n = MYFLT2LONG(*p->iSampleFormat);
-    if (UNLIKELY(n < 0 || n > 10))
-      return csound->InitError(csound, Str("diskin2: unknown sample format"));
-    sfinfo.format = diskin2_format_table[n];
+    if (n<0) {
+      n = -n;
+      if (UNLIKELY(n > 10))
+        return csound->InitError(csound, Str("diskin2: unknown sample format"));
+      sfinfo.format = diskin2_format_table[n];
+    }
     /* open file */
     /* FIXME: name can overflow with very long string */
     if (stringname==0){
@@ -2131,7 +2137,7 @@ static int32_t sndinset_(CSOUND *csound, SOUNDIN_ *p, int32_t stringname)
       /* skip initialisation if requested */
       if (*(p->iSkipInit) != FL(0.0))
         return OK;
-      fdclose(csound, &(p->fdch));
+      fd_close(csound, &(p->fdch));
     }
     /* set default format parameters */
     memset(&sfinfo, 0, sizeof(SF_INFO));
@@ -2144,9 +2150,12 @@ static int32_t sndinset_(CSOUND *csound, SOUNDIN_ *p, int32_t stringname)
         | (int32_t) FORMAT2SF(csound->oparms_.outformat);
     }
     else {
-      if (UNLIKELY(n < 0 || n > 10))
-        return csound->InitError(csound, Str("soundin: unknown sample format"));
-      sfinfo.format = diskin2_format_table[n];
+      if (n<0) {
+        n = -n;
+        if (UNLIKELY(n > 10))
+          return csound->InitError(csound, Str("soundin: unknown sample format"));
+        sfinfo.format = diskin2_format_table[n];
+      }
     }
     /* open file */
     /* FIXME: name can overflow with very long string */
