@@ -30,7 +30,10 @@
 #endif
 #include <lo/lo.h>
 #include <ctype.h>
-
+#ifndef WIN32
+  #include <sys/types.h>
+  #include <sys/socket.h>
+#endif
 //#define OSC_DEBUG
 
 /* structure for real time event */
@@ -1016,36 +1019,6 @@ static int32_t OSC_ahandler(const char *path, const char *types,
 }
 
 #include "arrays.h"
-#if 0
-static inline void tabensure(CSOUND *csound, ARRAYDAT *p, int32_t size)
-{
-    if (p->data==NULL || p->dimensions == 0 ||
-        (p->dimensions==1 && p->sizes[0] < size)) {
-      size_t ss;
-      if (p->data == NULL) {
-        CS_VARIABLE* var = p->arrayType->createVariable(csound, NULL);
-        p->arrayMemberSize = var->memBlockSize;
-      }
-      ss = p->arrayMemberSize*size;
-      if (p->data==NULL) {
-        p->data = (MYFLT*)csound->Calloc(csound, ss);
-        p->allocated = ss;
-      }
-      else if (ss > p->allocated) {
-        p->data = (MYFLT*) csound->ReAlloc(csound, p->data, ss);
-        p->allocated = ss;
-      }
-      if (p->dimensions==0) {
-        p->dimensions = 1;
-        p->sizes = (int32_t*)csound->Malloc(csound, sizeof(int32_t));
-      }
-      p->sizes[0] = size;
-    }
-    else {
-      p->sizes[0] = size;
-    }
-}
-#endif
 
 static int32_t OSC_alist_init(CSOUND *csound, OSCLISTENA *p)
 {
@@ -1065,7 +1038,7 @@ static int32_t OSC_alist_init(CSOUND *csound, OSCLISTENA *p)
                                            strlen((char*) p->dest->data) + 1);
     strcpy(p->c.saved_path, (char*) p->dest->data);
     /* check for a valid argument list */
-    tabensure(csound, p->args, n=strlen((char*) p->type->data));
+    tabinit(csound, p->args, n=strlen((char*) p->type->data));
     strcpy(p->c.saved_types, (char*) p->type->data);
     for (i = 0; i < n; i++) {
       switch (p->c.saved_types[i]) {
