@@ -45,6 +45,8 @@
 */
 
 #include "csdl.h"
+#include "arrays.h"
+#include "emugens_common.h"
 
 // -------------------------------------------------------------------------
 
@@ -68,9 +70,6 @@
         memset(&out[nsmps], '\0', early*sizeof(MYFLT));              \
     }                                                                \
 
-#define INITERR(m) (csound->InitError(csound, "%s", m))
-#define PERFERROR(m) (csound->PerfError(csound, &(p->h), "%s", m))
-
 
 /*
 
@@ -182,35 +181,6 @@ gaussians_init(uint32_t seed) {
   (gaussians[(uint32_t)(FastRandFloat(seed)*(GAUSSIANS_SIZE-1))])
 
 
-// from Opcodes/arrays.c, original name: tabensure.
-// This should be part of the API.
-
-#if 0
-static inline void
-arrayensure(CSOUND *csound, ARRAYDAT *p, int size) {
-    if (p->data==NULL || p->dimensions == 0 ||
-        (p->dimensions==1 && p->sizes[0] < size)) {
-      size_t ss;
-      if (p->data == NULL) {
-        CS_VARIABLE* var = p->arrayType->createVariable(csound, NULL);
-        p->arrayMemberSize = var->memBlockSize;
-      }
-      ss = p->arrayMemberSize*size;
-      if (p->data==NULL)
-        p->data = (MYFLT*)csound->Calloc(csound, ss);
-      else
-        p->data = (MYFLT*) csound->ReAlloc(csound, p->data, ss);
-      p->dimensions = 1;
-      p->sizes    = (int*)csound->Malloc(csound, sizeof(int));
-      p->sizes[0] = size;
-    }
-}
-#else
-#include "arrays.h"
-#define arrayensure tabensure
-#endif
-
-
 /*
 
    Integer phase oscillator with/without interpolation,
@@ -586,13 +556,11 @@ typedef struct {
     int updatearrays;
 } BEADSYNT;
 
-
 static int32_t
 beadsynt_init_common(CSOUND *csound, BEADSYNT *p) {
     FILTCOEFS *filtcoefs;
     int32_t *lphs;
     unsigned int c, count = p->count;
-
     MYFLT iphs = *p->iphs;
     MYFLT sr   = csound->GetSr(csound);
     p->inerr = 0;
@@ -1107,16 +1075,16 @@ tabrowcopy_init(CSOUND* csound, TABROWCOPY* p){
 
     int end = *p->iend;
     if(end > *p->inumcols)
-      return INITERR(Str("tabrowcopy: iend cannot be bigger than numcols"));
+        return INITERR(Str("tabrowcopy: iend cannot be bigger than numcols"));
 
     if(end == 0)
-      end = *p->inumcols;
+        end = *p->inumcols;
 
     p->end = end;
 
     int numcols_to_copy = (int)((end - *p->istart) / *p->istep);
     if (numcols_to_copy > p->tabdestlen)
-      return INITERR(Str("tabrowcopy: Destination table too small"));
+        return INITERR(Str("tabrowcopy: Destination table too small"));
 
     p->maxrow = (int)((p->tabsourcelen - *p->ioffset) / *p->inumcols) - 1;
     return OK;
@@ -1151,7 +1119,7 @@ tabrowcopyk(CSOUND* csound, TABROWCOPY* p) {
     int j    = 0;
 
     if(UNLIKELY(row < 0))
-      return PERFERROR(Str("tabrowcopy: krow cannot be negative"));
+      return PERFERR(Str("tabrowcopy: krow cannot be negative"));
 
     if (LIKELY(delta != 0)) {
       if (UNLIKELY(idx1+numcols > tabsourcelen)) {
@@ -1159,7 +1127,7 @@ tabrowcopyk(CSOUND* csound, TABROWCOPY* p) {
                        "krow: %f   row0: %d  idx1: %d  numcols: %d   "
                         "tabsourcelen: %d\n",
                         row, row0, idx1, numcols, tabsourcelen);
-        return PERFERROR(Str("tabrowcopy: tab off end"));
+        return PERFERR(Str("tabrowcopy: tab off end"));
       }
       for (i=idx0; i<idx1; i+=step) {
         x0 = tabsource[i];
@@ -1168,7 +1136,7 @@ tabrowcopyk(CSOUND* csound, TABROWCOPY* p) {
       }
     } else {
       if (UNLIKELY(idx1 > tabsourcelen))
-        return PERFERROR(Str("tabrowcopy: tab off end"));
+        return PERFERR(Str("tabrowcopy: tab off end"));
       for (i=idx0; i<idx1; i+=step) {
         tabdest[j++] = tabsource[i];
       }
@@ -1198,18 +1166,18 @@ tabrowcopyarr_init(CSOUND *csound, TABROWCOPYARR *p) {
     uint32_t end   = (uint32_t)*p->iend;
     uint32_t step  = (uint32_t)*p->istep;
     if(end > *p->inumcols)
-      return INITERR(Str("tabrowlin: iend cannot be bigger than numcols"));
+        return INITERR(Str("tabrowlin: iend cannot be bigger than numcols"));
     if(end == 0)
-      end = *p->inumcols;
+        end = (uint32_t) *p->inumcols;
     if(end <= start) {
-      return INITERR(Str("tabrowlin: end must be bigger than start"));
+        return INITERR(Str("tabrowlin: end must be bigger than start"));
     }
     p->end = end;
-    uint32_t numitems = (uint32_t)ceil((end - start) / (float)step);
+    uint32_t numitems = (uint32_t) (ceil((end - start) / (MYFLT)step));
     if(numitems <= 0) {
-      return INITERR(Str("tabrowlin: no items to copy"));
+        return INITERR(Str("tabrowlin: no items to copy"));
     }
-    arrayensure(csound, p->outarr, numitems);
+    tabinit(csound, p->outarr, numitems);
     p->numitems = numitems;
     p->maxrow = (p->tabsourcelen - *p->ioffset) / *p->inumcols - FL(2);
     return OK;
@@ -1227,20 +1195,24 @@ tabrowcopyarr_k(CSOUND *csound, TABROWCOPYARR *p) {
     uint32_t row0 = (uint32_t)row;
     MYFLT delta = row - row0;
     uint32_t tabsourcelen = p->tabsourcelen;
-    MYFLT *out = p->outarr->data;
-    MYFLT *tabsource = p->tabsource;
     MYFLT x0, x1;
 
     if(UNLIKELY(row < 0)) {
-      return PERFERROR(Str("krow cannot be negative"));
+      return PERFERR(Str("krow cannot be negative"));
     }
     // TODO : check maxrow
     uint32_t idx0 = offset + numcols * row0 + start;
     uint32_t idx1 = idx0 + (end-start);
+    uint32_t numitems = (uint32_t) (ceil((end - start) / (MYFLT)step));
+    ARRAY_ENSURESIZE(csound, p->outarr, numitems);
+
+    MYFLT *out = p->outarr->data;
+    MYFLT *tabsource = p->tabsource;
+
     uint32_t i, j = 0;
     if (LIKELY(delta != 0)) {
-      if (UNLIKELY(idx1+numcols >= tabsourcelen))
-        return PERFERROR(Str("tab off end"));
+    if (UNLIKELY(idx1+numcols >= tabsourcelen))
+        return PERFERR(Str("tab off end"));
       for (i=idx0; i<idx1; i+=step) {
         x0 = tabsource[i];
         x1 = tabsource[i + numcols];
@@ -1248,7 +1220,7 @@ tabrowcopyarr_k(CSOUND *csound, TABROWCOPYARR *p) {
       }
     } else {
       if (UNLIKELY(idx1 >= tabsourcelen))
-        return PERFERROR(Str("tab off end"));
+        return PERFERR(Str("tab off end"));
       for (i=idx0; i<idx1; i+=step) {
         out[j++] = tabsource[i];
       }
@@ -1284,9 +1256,8 @@ getrowlin_init(CSOUND *csound, GETROWLIN *p) {
     int step  = (int)*p->kstep;
     if (end < 1)
       end = p->inarr->sizes[1];
-    int numitems = (int)ceil((end - start) / (float)step);
-
-    arrayensure(csound, p->outarr, numitems);
+    int numitems = (int) (ceil((end - start) / (MYFLT)step));
+    tabinit(csound, p->outarr, numitems);
     p->numitems = numitems;
     return OK;
 }
@@ -1294,30 +1265,29 @@ getrowlin_init(CSOUND *csound, GETROWLIN *p) {
 static int32_t
 getrowlin_k(CSOUND *csound, GETROWLIN *p) {
     if(p->inarr->dimensions != 2)
-      return PERFERROR(Str("The input array should be a 2D array"));
+        return PERFERR(Str("The input array should be a 2D array"));
     int start = (int)*p->kstart;
     int end   = (int)*p->kend;
     int step  = (int)*p->kstep;
     if (end <= 0) {
-      end = p->inarr->sizes[1];
+        end = p->inarr->sizes[1];
     }
-    int numitems = (int)ceil((end - start) / (float)step);
+    int numitems = (int) (ceil((end - start) / (MYFLT)step));
     int numcols  = p->inarr->sizes[1];
     if(numitems > numcols)
-      return PERFERROR(Str("Asked to read too many items from a row"));
-    if(numitems > p->numitems) {
-      arrayensure(csound, p->outarr, numitems);
-      p->numitems = numitems;
-    }
+        return PERFERR(Str("Asked to read too many items from a row"));
+    ARRAY_ENSURESIZE(csound, p->outarr, numitems);
+
+    p->numitems = numitems;
     MYFLT row = *p->krow;
     int maxrow = p->inarr->sizes[0] - 1;
     if(UNLIKELY(row < 0))
-      return PERFERROR(Str("getrowlin: krow cannot be negative"));
+        return PERFERR(Str("getrowlin: krow cannot be negative"));
     if(UNLIKELY(row > maxrow)) {
-      csound->Message(csound, Str("getrowlin: row %.4f > maxrow %d, clipping\n"),
-                      row, maxrow);
-      row = maxrow;
-      // return PERFERROR(Str("getrowlin: exceeded maximum row"));
+        csound->Message(csound, Str("getrowlin: row %.4f > maxrow %d, clipping\n"),
+                        row, maxrow);
+        row = maxrow;
+        // return PERFERR(Str("getrowlin: exceeded maximum row"));
     }
     int row0    = (int)row;
     MYFLT delta = row - row0;
@@ -1330,16 +1300,16 @@ getrowlin_k(CSOUND *csound, GETROWLIN *p) {
     MYFLT x0, x1;
     int i, j = 0;
     if (LIKELY(delta != 0)) {
-      for (i=idx0; i<idx1; i+=step) {
-        x0 = in[i];
-        x1 = in[i + numcols];
-        out[j++] = x0 + (x1-x0)*delta;
-      }
+        for (i=idx0; i<idx1; i+=step) {
+            x0 = in[i];
+            x1 = in[i + numcols];
+            out[j++] = x0 + (x1-x0)*delta;
+        }
     }
     else {
-      for (i=idx0; i<idx1; i+=step) {
-        out[j++] = in[i];
-      }
+        for (i=idx0; i<idx1; i+=step) {
+            out[j++] = in[i];
+        }
     }
     return OK;
 }
@@ -1383,9 +1353,10 @@ static OENTRY localops[] = {
 
     // aout beadsynt kFreq[], kAmp[], kBw[],
     //               inumosc=-1, iflags=1, kfreq=1, kbw=1, ifn=-1, iphs=-1
-    {"beadsynt", S(BEADSYNT), TR, 3, "a", "k[]k[]k[]jpPPjj",
+    {"beadsynt", S(BEADSYNT), TR, 3, "a", ".[].[].[]jpPPjj",
      (SUBR)beadsynt_init_array, (SUBR)beadsynt_perf },
 
+
     // tabrowlin krow, ifnsrc, ifndest, inumcols,
     //                 ioffset=0, istart=0, iend=0, istep=1
     {"tabrowlin", S(TABROWCOPY), 0, 3, "", "kiiiooop",
