@@ -511,36 +511,6 @@ typedef struct _rawosc {
 } RAWOSC;
 
 #include "arrays.h"
-#if 0
-static inline void tabensure(CSOUND *csound, ARRAYDAT *p, int32_t size)
-{
-    if (p->data==NULL || p->dimensions == 0 ||
-        (p->dimensions==1 && p->sizes[0] < size)) {
-      size_t ss;
-      if (p->data == NULL) {
-        CS_VARIABLE* var = p->arrayType->createVariable(csound, NULL);
-        p->arrayMemberSize = var->memBlockSize;
-      }
-      ss = p->arrayMemberSize*size;
-      if (p->data==NULL) {
-        p->data = (MYFLT*)csound->Calloc(csound, ss);
-        p->allocated = ss;
-      }
-      else if (ss > p->allocated) {
-        p->data = (MYFLT*) csound->ReAlloc(csound, p->data, ss);
-        p->allocated = ss;
-      }
-      if (p->dimensions==0) {
-        p->dimensions = 1;
-        p->sizes = (int32_t*)csound->Malloc(csound, sizeof(int32_t));
-      }
-      p->sizes[0] = size;
-    }
-    else {
-      p->sizes[0] = size;
-    }
-}
-#endif
 
 static int32_t destroy_raw_osc(CSOUND *csound, void *pp) {
     RAWOSC *p = (RAWOSC *) pp;
@@ -596,7 +566,7 @@ static int32_t init_raw_osc(CSOUND *csound, RAWOSC *p)
     }
 
     csound->RegisterDeinitCallback(csound, (void *) p, destroy_raw_osc);
-    tabensure(csound, p->sout,2);
+    tabinit(csound, p->sout,2);
 
   return OK;
 }
