@@ -300,7 +300,8 @@ static const char *longUsageList[] = {
   Str_noop("--vbr-quality=Ft        set quality of variable bit-rate compression"),
   Str_noop("--devices[=in|out]      list available audio devices and exit"),
   Str_noop("--midi-devices[=in|out] list available MIDI devices and exit"),
-  Str_noop("--get-system-sr         print system sr and exit"),
+  Str_noop("--get-system-sr         print system sr and exit, requires realtime audio output (e.g. -odac) to be defined first)"),
+  Str_noop("--use-system-sr         print system sr and use realtime audio output (e.g. -odac) to be defined first"),
   Str_noop("--ksmps=N               override ksmps"),
   Str_noop("--fftlib=N              actual FFT lib to use (FFTLIB=0, "
                                    "PFFFT = 1, vDSP =2)"),
@@ -1161,7 +1162,7 @@ static int decode_long(CSOUND *csound, char *s, int argc, char **argv)
       csound->info_message_request = 1;
       return 1;
       }
-    else if (!(strncmp(s, "get-system-sr",13))){
+    else if (!(strncmp(s, "get-system-sr",13)) ){
       if (O->outfilename &&
           !(strncmp(O->outfilename, "dac",3))) {
         /* these are default values to get the
@@ -1180,6 +1181,11 @@ static int decode_long(CSOUND *csound, char *s, int argc, char **argv)
       csound->info_message_request = 1;
       return 1;
     }
+    else if(!strncmp(s, "use-system-sr",13)){
+      if (O->sr_override == FL(0.0))
+          O->sr_override = FL(-1.0);
+      return 1;
+    }
     else if (!(strcmp(s, "aft-zero"))){
       csound->aftouch = 0;
       return 1;
